/******************************************************************************
 *                                                                            *
 * Copyright (c) 2023-2024, Enno Boland <g@s01.de>                            *
 *                                                                            *
 * Redistribution and use in source and binary forms, with or without         *
 * modification, are permitted provided that the following conditions are     *
 * met:                                                                       *
 *                                                                            *
 * * Redistributions of source code must retain the above copyright notice,   *
 *   this list of conditions and the following disclaimer.                    *
 * * Redistributions in binary form must reproduce the above copyright        *
 *   notice, this list of conditions and the following disclaimer in the      *
 *   documentation and/or other materials provided with the distribution.     *
 *                                                                            *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS    *
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,  *
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR     *
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR          *
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,      *
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,        *
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR         *
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF     *
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING       *
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS         *
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.               *
 *                                                                            *
 ******************************************************************************/

/**
 * @author       Enno Boland (mail@eboland.de)
 * @file         nasty.c
 */

#include "../libsqsh/include/sqsh_archive_private.h"
#include "../libsqsh/include/sqsh_file_private.h"
#include <utest.h>

UTEST(nast, nasty_sqfs) {
	int rv = 0;
	// generated by squashfs-tools-ng's mknasty.
	unsigned char nasty_sqfs[4096] = {
			0x68, 0x73, 0x71, 0x73, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00,
			0x0c, 0x00, 0x50, 0x02, 0x01, 0x00, 0x04, 0x00, 0x00, 0x00, 0x48,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc2, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0xba, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x60, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x8e, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x2c, 0x00, 0x78,
			0xda, 0x63, 0x62, 0x58, 0xc2, 0xc8, 0x00, 0x05, 0x30, 0xc6, 0x7f,
			0x20, 0x00, 0xd1, 0x96, 0xac, 0x60, 0xcc, 0xc8, 0x84, 0xa4, 0x86,
			0x09, 0x87, 0x1a, 0x46, 0x86, 0xb7, 0x70, 0x35, 0xcc, 0x50, 0x9a,
			0x05, 0x88, 0x75, 0xa0, 0x6c, 0x00, 0xc5, 0x4d, 0x0b, 0x66, 0x24,
			0x00, 0x78, 0xda, 0x63, 0x64, 0x80, 0x00, 0x46, 0x28, 0xcd, 0x04,
			0x64, 0xe9, 0xe9, 0xa9, 0x00, 0x49, 0x26, 0x06, 0x2e, 0x06, 0xfd,
			0xd4, 0x92, 0x64, 0xfd, 0x82, 0xc4, 0xe2, 0xe2, 0xf2, 0x14, 0x00,
			0x22, 0xdf, 0x04, 0xbf, 0x04, 0x80, 0x00, 0x00, 0x00, 0x00, 0xb4};
	struct SqshConfig config = {
			.source_size = sizeof(nasty_sqfs),
			.source_mapper = sqsh_mapper_impl_static,
	};

	struct SqshArchive *archive = sqsh_archive_open(nasty_sqfs, &config, &rv);
	ASSERT_EQ(0, rv);
	ASSERT_NE(NULL, archive);
	const struct SqshSuperblock *superblock = sqsh_archive_superblock(archive);
	uint64_t inode_ref = sqsh_superblock_inode_root_ref(superblock);

	struct SqshFile *root_dir = sqsh_open_by_ref(archive, inode_ref, &rv);
	ASSERT_NE(0, rv);
	ASSERT_EQ(NULL, root_dir);

	sqsh_close(root_dir);
	sqsh_archive_close(archive);
}

UTEST_MAIN()
